using System;
using System.Collections.Generic;
using System.Drawing.Design;
using System.Windows.Forms;
using System.IO;
using System.Xml;
using System.Linq;
using System.Collections;
using GrapeCity.ActiveReports;
using GrapeCity.ActiveReports.Controls;
using GrapeCity.ActiveReports.SectionReportModel;
using GrapeCity.ActiveReports.Document.Section;
using GrapeCity.ActiveReports.Design;
using GrapeCity.ActiveReports.Data;

namespace GrapeCity.ActiveReports.Samples.Professional.EndUserDesigner
{
    public partial class EndUserDesignerMainForm : Form
    {
        string selectedsubreport = "";
        string selectedsection = "";
        ToolStripComboBox reportsCombo = new ToolStripComboBox("ReportsCombo");

        //collection of report layouts.....
        Dictionary<string, System.IO.MemoryStream> ReportLayouts = new Dictionary<string, System.IO.MemoryStream>();

        public EndUserDesignerMainForm()
        {
            InitializeComponent();

            //Create new report instance and assign to Report Explorer
            //Note:  Assigning the ToolBox to the designer before calling NewReport
            // will automaticly add the default controls to the toolbox in a group called
            // "ActiveReports 6"
            arDesigner.Toolbox = arToolbox;
            arReportExplorer.ReportDesigner = arDesigner;

            // Add Menu and CommandBar to Form
            ToolStrip menuStrip = arDesigner.CreateToolStrips(DesignerToolStrips.Menu)[0];

            ToolStripDropDownItem fileMenu = (ToolStripDropDownItem)menuStrip.Items[0];

            // Add a Export to RDF and Exit command to the File menu
            fileMenu.DropDownItems.Add(new ToolStripButton("Export to RDF...", null, DoExportRdf));
            fileMenu.DropDownItems.Add(new ToolStripSeparator());
            fileMenu.DropDownItems.Add(new ToolStripButton("Exit", null, OnExit));

            #region Adding combobox for report layouts and a button to save report to memory........
            reportsCombo.ToolTipText = "Report Layouts in Memory";
            reportsCombo.SelectedIndexChanged += new EventHandler(reportsCombo_SelectedIndexChanged);
            ToolStripButton toolSaveToMemory = new ToolStripButton("Save To Memory");
            toolSaveToMemory.Click += new EventHandler(toolSaveToMemory_Click);
            ToolStrip reporttool = arDesigner.CreateToolStrips(DesignerToolStrips.Report)[0];
            reporttool.Items.Add(reportsCombo);
            reporttool.Items.Add(toolSaveToMemory);
            AppendToolStrips(1, new ToolStrip[] { reporttool });
            AppendToolStrips(0, new ToolStrip[] { menuStrip });
            #endregion


            AppendToolStrips(1, arDesigner.CreateToolStrips(
                DesignerToolStrips.Edit,
                DesignerToolStrips.Undo,
                DesignerToolStrips.Zoom));

            AppendToolStrips(2, arDesigner.CreateToolStrips(
                DesignerToolStrips.Format, DesignerToolStrips.Layout));

            //Fill Toolbox
            LoadTools(this.arToolbox);
            // Activate default group on the toolbox
            this.arToolbox.SelectedCategory = "Section Reports";
        }


        #region Code to save the report layout to memory on SaveToMemory toostrip button click
        void toolSaveToMemory_Click(object sender, EventArgs e)
        {
            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            ((SectionReport)arDesigner.Report).SaveLayout(stream);
        SHOWDIALOGAGAIN:
            SaveDialog LayoutSaveDialog = new SaveDialog();
            if (LayoutSaveDialog.ShowDialog() == DialogResult.OK)
            {
                if (reportsCombo.Items.Contains(LayoutSaveDialog.ReportName) == true)
                {
                    if (MessageBox.Show("A report is already existed with the same name, Do you want to overwrite?", "ReportName exists", MessageBoxButtons.YesNo) == DialogResult.Yes)
                    {
                        reportsCombo.Items.Remove(LayoutSaveDialog.ReportName);
                        ReportLayouts.Remove(LayoutSaveDialog.ReportName);
                        ReportLayouts.Add(LayoutSaveDialog.ReportName, stream);
                        reportsCombo.Items.Add(LayoutSaveDialog.ReportName);
                    }
                    else
                    {
                        goto SHOWDIALOGAGAIN;
                    }
                }
                else
                {
                    ReportLayouts.Add(LayoutSaveDialog.ReportName, stream);
                    reportsCombo.Items.Add(LayoutSaveDialog.ReportName);
                }
            }

        }

        #endregion


        //Loading report layout from memory to designer when reports combobox toolstrip item is changed.....
        void reportsCombo_SelectedIndexChanged(object sender, EventArgs e)
        {
            arDesigner.NewReport();
            if (((ToolStripComboBox)sender).SelectedItem != null)
            {
                System.IO.MemoryStream reportstream = ReportLayouts[((ToolStripComboBox)sender).SelectedItem.ToString()];
                reportstream.Position = 0;
                ((SectionReport)arDesigner.Report).LoadLayout(reportstream);
            }
        }


        private void AppendToolStrips(int row, IList<ToolStrip> toolStrips)
        {
            ToolStripPanel panel = toolStripContainer1.TopToolStripPanel;
            for (int i = toolStrips.Count; --i >= 0; )
            {
                panel.Join(toolStrips[i], row);
            }
        }

        private static void LoadTools(IToolboxService toolbox)
        {
            //Add Data Providers
            foreach (Type type in new Type[]
				{
					typeof (System.Data.DataSet),
					typeof (System.Data.DataView),
					typeof (System.Data.OleDb.OleDbConnection),
					typeof (System.Data.OleDb.OleDbDataAdapter),
					typeof (System.Data.Odbc.OdbcConnection),
					typeof (System.Data.Odbc.OdbcDataAdapter),
					typeof (System.Data.SqlClient.SqlConnection),
					typeof (System.Data.SqlClient.SqlDataAdapter)
				})
            {
                toolbox.AddToolboxItem(new ToolboxItem(type), "Data");
            }
        }

        /// <summary>
        /// OnExit
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnExit(object sender, EventArgs e)
        {
            Close();
        }

        private void DoExportRdf(object sender, EventArgs e)
        {
            SaveFileDialog saveRdfDialog = new SaveFileDialog();
            saveRdfDialog.Filter = "RDF files (*.rdf)|*.rdf|All files (*.*)|*.*";
            if (saveRdfDialog.ShowDialog() == DialogResult.OK)
            {
                ((SectionReport)arDesigner.Report).Run(false);
                ((SectionReport)arDesigner.Report).Document.Save(saveRdfDialog.FileName);
            }
        }

        #region Designer Events

        private void arDesigner_SelectionChanged()
        {
            selectedsubreport = "";
            selectedsection = "";
            List<string> selectionItems = new List<string>();
            if (arDesigner.Selection != null)
            {
                foreach (object item in arDesigner.Selection)
                {
                    string itemName;
                    if (item is Section)
                    {
                        itemName = ((Section)item).Name;
                        selectedsection = itemName;
                    }
                    else if (item is ARControl)
                    {
                        itemName = ((ARControl)item).Name;
                        if (item is SubReport)
                        {
                            selectedsubreport = itemName;
                        }
                    }
                    else if (item is Field)
                        itemName = ((Field)item).Name;
                    else if (item is Parameter)
                        itemName = ((Parameter)item).Key;
                    else if (item is SectionReport)
                        itemName = ((SectionReport)item).Document.Name;
                    else
                        continue;

                    selectionItems.Add(itemName);
                }
            }

            if (selectionItems.Count > 0)
                toolStripStatusLabel2.Text = "Current Selection: " + string.Join(", ", selectionItems.ToArray());
            else
                toolStripStatusLabel2.Text = "No Selection";
        }


        #endregion

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        private static void Main()
        {
            Application.Run(new EndUserDesignerMainForm());
        }


        #region Manipulate layouts for all subreports accordingly and decoding them to a string
        private void arDesigner_LayoutChanging(object sender, LayoutChangingArgs e)
        {
            //This code is for manipulate layouts for all subreports accordingly and decoding them to a string and assigning in userdata......
            if (e.Type == LayoutChangeType.ControlAdd)
            {
                if (e.NewValue.GetType() == typeof(GrapeCity.ActiveReports.SectionReportModel.SubReport))
                {
                    ReportSelectDialog selectdialog = new ReportSelectDialog(reportsCombo.Items);

                    if (selectdialog.ShowDialog() == DialogResult.OK)
                    {
                        System.IO.MemoryStream stream = new MemoryStream();
                        if (selectdialog.reportstream != null)
                        {
                            stream = selectdialog.reportstream;
                        }
                        else if (selectdialog.SubreportName != null)
                        {
                            stream = ReportLayouts[selectdialog.SubreportName];
                        }
                        else
                        {
                            stream = null;
                        }
                        if (stream != null)
                        {
                            stream.Seek(0, SeekOrigin.Begin);
                            StreamReader reader = new StreamReader(stream);
                            string rptlayoutstring = reader.ReadToEnd();
                            rptlayoutstring = rptlayoutstring.Replace("encoding=\"utf-16\"", "encoding=\"utf-8\"");
                            ((SectionReport)arDesigner.Report).UserData += "~SubreportName:" + e.Name + "~Section:" + e.NewSectionName + "~SubreportData:" + rptlayoutstring;

                            if (((SectionReport)arDesigner.Report).Script.Contains("::subreport assignment started::") == false)
                            {

                                if (((SectionReport)arDesigner.Report).Script.Contains("public void ActiveReport_ReportStart()") == false)
                                {
                                    ((SectionReport)arDesigner.Report).Script += Resource1.SubRepAssignWithReportStart;
                                }
                                else
                                {
                                    int startindex = ((SectionReport)arDesigner.Report).Script.IndexOf("public void ActiveReport_ReportStart()");
                                    int startindex2 = ((SectionReport)arDesigner.Report).Script.Substring(startindex, ((SectionReport)arDesigner.Report).Script.Length - (startindex + 1)).IndexOf("{");
                                    ((SectionReport)arDesigner.Report).Script = ((SectionReport)arDesigner.Report).Script.Insert(startindex + startindex2 + 2, Resource1.SubRepAssignWithoutReportStart);
                                }
                            }
                        }
                    }
                }
                else
                {

                }

            }

            else if (e.Type == LayoutChangeType.ControlDelete)
            {
                if (((SectionReport)arDesigner.Report).UserData != null)
                {
                    IList<SubreportInternalData> col = SubreportInternalData.GetSubreportIntenalDataCollection(((SectionReport)arDesigner.Report).UserData.ToString());
                    var item = col.Where(a => a.SubreportName == e.Name);
                    if (item.Count() > 0)
                    {
                        col.Remove((SubreportInternalData)item.ToList()[0]);
                        ((SectionReport)arDesigner.Report).UserData = SubreportInternalData.GetSubreportIntenalWholeData(col);
                    }
                }
            }
            else if (e.Type == LayoutChangeType.ControlMove)
            {
                if (e.OldSectionName != e.NewSectionName)
                {
                    IList<SubreportInternalData> col = SubreportInternalData.GetSubreportIntenalDataCollection(((SectionReport)arDesigner.Report).UserData.ToString());
                    var items = col.Where(a => a.SubreportName == e.Name);
                    if (items.Count() > 0)
                    {
                        SubreportInternalData item = ((SubreportInternalData)items.ToList()[0]);
                        SubreportInternalData newitem = new SubreportInternalData(e.NewSectionName, item.SubreportName, item.SubReportData);
                        col.Remove(item);
                        col.Add(newitem);
                        ((SectionReport)arDesigner.Report).UserData = SubreportInternalData.GetSubreportIntenalWholeData(col);
                    }
                }
            }
            else if (e.Type == LayoutChangeType.SectionDelete)
            {
                IList<SubreportInternalData> col = SubreportInternalData.GetSubreportIntenalDataCollection(((SectionReport)arDesigner.Report).UserData.ToString());
                var items = col.Where(a => a.SectionName == e.Name);
                items.ToList().Clear();
                for (int i = 0; i < items.Count(); i++)
                {
                    SubreportInternalData item = ((SubreportInternalData)items.ToList()[0]);
                    col.Remove(item);
                    i--;
                }
                ((SectionReport)arDesigner.Report).UserData = SubreportInternalData.GetSubreportIntenalWholeData(col);
            }
            else
            {

            }

        }
        #endregion


        #region Make changes in user data when subreport or section name is changed
        private void arPropertyGrid_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            //this code is to make necessary changes in the userdata when subreport or section name is changed....
            if (e.ChangedItem.Label.ToString() == "(Name)")
            {
                if (((SectionReport)arDesigner.Report).UserData != null)
                {
                    if (selectedsubreport != "")
                    {
                        IList<SubreportInternalData> col = SubreportInternalData.GetSubreportIntenalDataCollection(((SectionReport)arDesigner.Report).UserData.ToString());
                        var items = col.Where(a => a.SubreportName == e.OldValue.ToString());
                        if (items.Count() > 0)
                        {
                            SubreportInternalData item = ((SubreportInternalData)items.ToList()[0]);
                            SubreportInternalData newitem = new SubreportInternalData(item.SectionName, e.ChangedItem.Value.ToString(), item.SubReportData);
                            col.Remove(item);
                            col.Add(newitem);
                            ((SectionReport)arDesigner.Report).UserData = SubreportInternalData.GetSubreportIntenalWholeData(col);
                        }
                    }
                    else if (selectedsection != "")
                    {
                        IList<SubreportInternalData> col = SubreportInternalData.GetSubreportIntenalDataCollection(((SectionReport)arDesigner.Report).UserData.ToString());
                        var items = col.Where(a => a.SectionName == e.OldValue.ToString());
                        for (int i = 0; i < items.Count(); i++)
                        {
                            SubreportInternalData item = ((SubreportInternalData)items.ToList()[i]);
                            SubreportInternalData newitem = new SubreportInternalData(e.ChangedItem.Value.ToString(), item.SubreportName, item.SubReportData);
                            col.Remove(item);
                            col.Add(newitem);
                            ((SectionReport)arDesigner.Report).UserData = SubreportInternalData.GetSubreportIntenalWholeData(col);
                        }

                    }
                }

            }
        }
    }
        #endregion


    #region Handling decoding and encoding subreport layout collection in userdata

    public class SubreportInternalData
    {
        public static IList<SubreportInternalData> GetSubreportIntenalDataCollection(string wholedata)
        {
            IList<SubreportInternalData> col = new List<SubreportInternalData>();

            string[] subreports = wholedata.Split(new string[] { "~SubreportName:" }, System.StringSplitOptions.RemoveEmptyEntries);
            if (subreports.Length > 0)
            {
                for (int i = 0; i < subreports.Length; i++)
                {
                    SubreportInternalData sid = new SubreportInternalData();
                    string[] sections = subreports[i].Split(new string[] { "~Section:" }, System.StringSplitOptions.RemoveEmptyEntries);
                    sid.SubreportName = sections[0];
                    string[] subreportdata = sections[1].Split(new string[] { "~SubreportData:" }, System.StringSplitOptions.RemoveEmptyEntries);
                    sid.SectionName = subreportdata[0];
                    if (subreportdata.Count() > 1)
                    {
                        string ud = subreportdata[1];
                        sid.SubReportData = ud.Replace("&lt;", "<").Replace("&gt;", ">");
                    }

                    col.Add(sid);
                }
            }
            return col;
        }
        public static string GetSubreportIntenalWholeData(IList<SubreportInternalData> subreportinternaldatacol)
        {
            string wholedata = "";
            if (subreportinternaldatacol.Count > 0)
            {
                for (int i = 0; i < subreportinternaldatacol.Count; i++)
                {
                    wholedata += "~SubreportName:" + subreportinternaldatacol[i].SubreportName + "~Section:" + subreportinternaldatacol[i].SectionName + "~SubreportData:" + subreportinternaldatacol[i].SubReportData;
                }
            }
            return wholedata;
        }


        public string SectionName { get; set; }
        public string SubreportName { get; set; }
        public string SubReportData { get; set; }
        public SubreportInternalData(string sectionname, string subreportname, string subreportdata)
        {
            SectionName = sectionname;
            SubreportName = subreportname;
            SubReportData = subreportdata;
        }
        public SubreportInternalData()
        {

        }
    }
    #endregion
}